package com.gic.haoban.manage.service.service.out.impl;

import java.util.ArrayList;
import java.util.List;

import com.alibaba.fastjson.JSON;
import com.gic.haoban.base.api.common.Constant;
import com.gic.haoban.manage.service.entity.TabHaobanDepartment;
import com.gic.haoban.manage.service.service.DepartmentService;
import com.gic.wechat.api.dto.qywx.DepartmentDTO;
import com.gic.wechat.api.service.qywx.QywxDepartmentApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.hutool.core.collection.CollectionUtil;

import com.gic.commons.util.EntityUtil;
import com.gic.haoban.manage.api.dto.QywxCorpInfoSimpleDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.dto.YwWxEnterpriseDTO;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseApiService;
import com.gic.haoban.manage.service.dao.mapper.WxEnterpriseMapper;
import com.gic.haoban.manage.service.entity.TabHaobanWxApplication;
import com.gic.haoban.manage.service.entity.TabHaobanWxEnterprise;
import com.gic.haoban.manage.service.service.WxApplicationService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;

@Service
public class WxEnterpriseApiServiceImpl implements WxEnterpriseApiService {
	private static final Logger log = LogManager.getLogger(WxEnterpriseApiServiceImpl.class);
	@Autowired
	private WxEnterpriseService wxEnterpriseService;
	@Autowired
	private StaffApiService staffApiService;
	@Autowired
	private WxEnterpriseMapper wxEnterpriseMapper;
	@Autowired
	private WxApplicationService wxApplicationService;
	@Autowired
	private QywxDepartmentApiService qywxDepartmentApiService;
	@Autowired
	private DepartmentService departmentService;
	
	@Override
	public void callbackReviceEnterprise(QywxCorpInfoSimpleDTO dto) {
		log.info("企业微信授权信息：{}", JSON.toJSONString(dto));
		WxEnterpriseDTO wxDTO = EntityUtil.changeEntityByJSON(WxEnterpriseDTO.class, dto);
		
		
		String wxEnterpriseId = wxEnterpriseService.add(wxDTO);
		String userId = dto.getUserid();
		String name = dto.getName();
		if(StringUtils.isNotBlank(userId)) {
			staffApiService.wxGetAdd(userId,wxEnterpriseId);
		}
		
		String siteId = dto.getSuiteId();
		String agentId = dto.getAgentid();
		String agentName = dto.getAgentName();
		String permanentCode = dto.getPermanentCode();
		String applicationName = dto.getName();
		
		TabHaobanWxApplication tab = wxApplicationService.selectBySiteIdAndWxEnterpriseId(siteId,wxEnterpriseId);
		if(tab == null) {
			tab = new TabHaobanWxApplication();
			tab.setAgentId(agentId);
			tab.setSiteId(siteId);
			tab.setAgentName(agentName);
			tab.setName(applicationName);
			tab.setPermanentCode(permanentCode);
			tab.setWxEnterpriseId(wxEnterpriseId);
			wxApplicationService.insert(tab);
		}else {
			tab.setPermanentCode(permanentCode);
			wxApplicationService.update(tab);
		}
		log.info("企业微信授权处理完成");
		if(dto.getLevel() != null && dto.getLevel() == 3){ //如果是通讯录应用拉取所有部门初始化
			List<DepartmentDTO> list = this.qywxDepartmentApiService.listDepartment(dto.getCorpid(), dto.getSuiteId(), null);
			log.info("企业所有部门数据：{}", JSON.toJSONString(list));
			if(CollectionUtils.isNotEmpty(list)){
				this.addAllDepartment(list, 1, wxEnterpriseId);
			}
		}
	}

	@Override
	public List<YwWxEnterpriseDTO> list() {
		List<TabHaobanWxEnterprise>  list = wxEnterpriseMapper.list();
		if(CollectionUtil.isEmpty(list)){
			return new ArrayList<YwWxEnterpriseDTO>();
		}
		List<YwWxEnterpriseDTO>  resultList = EntityUtil.changeEntityListByJSON(YwWxEnterpriseDTO.class, list);
		return resultList;
	}

	@Override
	public WxEnterpriseDTO getOne(String wxEnterpriseId) {
		TabHaobanWxEnterprise tab = wxEnterpriseMapper.selectByPrimaryKey(wxEnterpriseId);
		WxEnterpriseDTO wxDTO = EntityUtil.changeEntityByJSON(WxEnterpriseDTO.class, tab);
		return wxDTO;
	}

	@Override
	public WxEnterpriseDTO getEnterpriseBycorpId(String corpId) {
		TabHaobanWxEnterprise enterprise = this.wxEnterpriseService.getEnterpriseBycorpId(corpId);
		return EntityUtil.changeEntityByJSON(WxEnterpriseDTO.class, enterprise);
	}


	private void addAllDepartment(List<DepartmentDTO> list, Integer parentId, String wxEnterpriseId){
		int level = 1;
		String chainId = "0";
		String chainName = "0";
		if(parentId != 1){
			TabHaobanDepartment parent = this.departmentService.getByWxId(parentId + "");
			if(parent != null){
				level = parent.getLevel() + 1;
				chainId = parent.getChainId() + Constant.ID_SEPARATOR + parent.getDepartmentId();
				chainName = parent.getChainName() + Constant.NAME_SEPARATOR + parent.getChainName();
			}
		}
		for(DepartmentDTO departmentDTO : list){
			if(parentId == departmentDTO.getParentid()){
				com.gic.haoban.manage.api.dto.DepartmentDTO dto = new com.gic.haoban.manage.api.dto.DepartmentDTO();
				dto.setDepartmentName(departmentDTO.getName());
				dto.setLevel(level);
				dto.setWxEnterpriseId(wxEnterpriseId);
				dto.setWxDepartmentId(departmentDTO.getId()+"");
				dto.setParentDepartmentId(parentId+"");
				dto.setChainId(chainId);
				dto.setChainName(chainName);
				this.departmentService.add(dto);
				this.addAllDepartment(list, departmentDTO.getId(), wxEnterpriseId);
			}
		}
	}

}
